/*
 *  Copyright 2001-2010 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of Graph Game.
 *
 *  Graph Game is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Graph Game is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Graph Game.  If not, see <http://www.gnu.org/licenses/>.
 */
package kryshen.graphg;

class Edge {
    Node node1;
    Node node2;

    boolean shorted = false;
    boolean selected = false;
    boolean enabled = true;
    boolean marked = false;
    
    float value = 0f;

    boolean isVertical() {
	return node1.x == node2.x;
    }

    int getVectorX() {
	return node2.x - node1.x;
    }
    
    int getVectorY() {
	return node2.y - node1.y;
    }
    
    private float getK() {
        if (isVertical()) {
            return -1;
        } else {
            return (float) getVectorY() / (float) getVectorX();
        }
    }

    private float getB() {
        if (isVertical()) {
            return node1.x;
        } else {
            return node1.y - node1.x * getK();
        }
    }

    public boolean crosses(Edge other) {
        if (other.isVertical()) {
            if (isVertical()) {
                return false;
            } else {
                return other.crosses(this);
            }
        }

        float x;
        if (isVertical()) {
            x = node1.x;
        } else {
            x = (other.getB() - getB()) / (getK() - other.getK());
        }
        float y = other.getK() * x + other.getB();

        //System.out.println("k="+getK()+"; b="+getB()+"; x="+x+"; y="+y);

        if (x >= Math.min(node1.x, node2.x) &
                x <= Math.max(node1.x, node2.x) &
                x >= Math.min(other.node1.x, other.node2.x) &
                x <= Math.max(other.node1.x, other.node2.x) &
                y >= Math.min(node1.y, node2.y) &
                y <= Math.max(node1.y, node2.y) &
                y >= Math.min(other.node1.y, other.node2.y) &
                y <= Math.max(other.node1.y, other.node2.y)) {
            return true;
        } else {
            return false;
        }
    }


    
    Edge(Node node1, Node node2) {
	this.node1 = node1; this.node2 = node2;
    }

    public void  swapNodes() {
	Node tmp = node1;
	node1 = node2;
	node2 = tmp;
    }

    public boolean equals(Object o) {
	if (!(o instanceof Edge)) return false;

	Edge edge = (Edge)o;
	return ((node1.equals(edge.node1) && node2.equals(edge.node2)) ||
		(node1.equals(edge.node2) && node2.equals(edge.node1)));
    }

    public String toString() {
	return "(" + node1.x + "; " + node1.y + ") - (" + 
	    node2.x + "; " + node2.y + ")";
    }

    Node getAnotherNode(Node node) {
        if (node.equals(this.node1)) {
            return this.node2;
        } else {
            return this.node1;
        }
    }

    MultiNode getAnotherNode(MultiNode gnode) {
        if (gnode.equals(this.node1.multiNode)) {
            return this.node2.multiNode;
        } else {
            return this.node1.multiNode;
        }
    }

    MultiNode incidentNode(Edge link) {
        if (node1.multiNode == link.node1.multiNode ||
                node1.multiNode == link.node2.multiNode) {
            return node1.multiNode;
        }

        if (node2.multiNode == link.node1.multiNode ||
                node2.multiNode == link.node2.multiNode) {
            return node2.multiNode;
        }

        return null;
    }

    void shorten() {
        if (shorted) {
            return;
        }

        shorted = true;
        node1.connectedEdges.addElement(this);
        node2.connectedEdges.addElement(this);
        node1.multiNode.connect(node2.multiNode);
    }
}
